'''
Settings of some constant values.
'''

LIMIT = 1

DEPTH = 1

GETLEADER_TIMEOUT = 5
# Whether to convert 'dateTime' type to string with format  1970-01-01 08:00:00
DateTimeIs2Str = True
# Whether to convert 'timestamp' type to string with format  1970-01-01 08:00:00  (in timezone of the client end)
TimestampIs2Str = False
# Whether to use 7-digit for the accuracy of 'float' type
Float_Accuracy = True