
class SystemPrivileges:
    '''
        Data class for system privileges.
    '''

    TRUNCATE = 'TRUNCATE'
    COMPACT = 'COMPACT'
    CREATE_GRAPH = 'CREATE_GRAPH'
    SHOW_GRAPH = 'SHOW_GRAPH'
    DROP_GRAPH = 'DROP_GRAPH'
    ALTER_GRAPH = 'ALTER_GRAPH'
    MOUNT_GRAPH = 'MOUNT_GRAPH'
    UNMOUNT_GRAPH = 'UNMOUNT_GRAPH'
    TOP = 'TOP'
    KILL = 'KILL'
    STAT = 'STAT'
    SHOW_POLICY = 'SHOW_POLICY'
    CREATE_POLICY = 'CREATE_POLICY'
    DROP_POLICY = 'DROP_POLICY'
    ALTER_POLICY = 'ALTER_POLICY'
    SHOW_USER = 'SHOW_USER'
    CREATE_USER = 'CREATE_USER'
    DROP_USER = 'DROP_USER'
    ALTER_USER = 'ALTER_USER'
    SHOW_PRIVILEGE = 'SHOW_PRIVILEGE'


class GraphPrivileges:
    '''
        Data class for graphset privileges.
    '''
    TEMPLATE = "TEMPLATE"
    KHOP = "KHOP"
    AB = "AB"
    SPREAD = "SPREAD"
    AUTONET = "AUTONET"
    FIND = "FIND"
    INSERT = "INSERT"
    EXPORT = "EXPORT"
    UPSERT = "UPSERT"
    UPDATE = "UPDATE"
    DELETE = "DELETE"
    CREATE_SCHEMA = "CREATE_SCHEMA"
    DROP_SCHEMA = "DROP_SCHEMA"
    ALTER_SCHEMA = "ALTER_SCHEMA"
    SHOW_SCHEMA = "SHOW_SCHEMA"
    CREATE_PROPERTY = "CREATE_PROPERTY"
    DROP_PROPERTY = "DROP_PROPERTY"
    ALTER_PROPERTY = "ALTER_PROPERTY"
    SHOW_PROPERTY = "SHOW_PROPERTY"
    CREATE_FULLTEXT = "CREATE_FULLTEXT"
    DROP_FULLTEXT = "DROP_FULLTEXT"
    SHOW_FULLTEXT = "SHOW_FULLTEXT"
    CREATE_INDEX = "CREATE_INDEX"
    DROP_INDEX = "DROP_INDEX"
    SHOW_INDEX = "SHOW_INDEX"
    LTE = "LTE"
    UFE = "UFE"
    CLEAR_TASK = "CLEAR_TASK"
    STOP_TASK = "STOP_TASK"
    PAUSE_TASK = "PAUSE_TASK"
    RESUME_TASK = "RESUME_TASK"
    SHOW_TASK = "SHOW_TASK"
    ALGO = "ALGO"
    SHOW_ALGO = "SHOW_ALGO"