# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ultipa.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cultipa.proto\x12\x06ultipa\"\"\n\x12HelloUltipaRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"C\n\x10HelloUltipaReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x84\x01\n\nUqlRequest\x12\x0b\n\x03uql\x18\x01 \x01(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\r\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12\x11\n\tuser_name\x18\x04 \x01(\t\x12\x12\n\nthread_num\x18\x05 \x01(\r\x12\n\n\x02tz\x18\x06 \x01(\t\x12\x11\n\ttz_offset\x18\x07 \x01(\t\"E\n\x0bResultAlias\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\'\n\x0bresult_type\x18\x02 \x01(\x0e\x32\x12.ultipa.ResultType\"\x9a\x03\n\x08UqlReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x17\n\x0ftotal_time_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\"\n\x05\x61lias\x18\x04 \x03(\x0b\x32\x13.ultipa.ResultAlias\x12 \n\x05paths\x18\x05 \x03(\x0b\x32\x11.ultipa.PathAlias\x12 \n\x05nodes\x18\x06 \x03(\x0b\x32\x11.ultipa.NodeAlias\x12 \n\x05\x65\x64ges\x18\x07 \x03(\x0b\x32\x11.ultipa.EdgeAlias\x12 \n\x05\x61ttrs\x18\x08 \x03(\x0b\x32\x11.ultipa.AttrAlias\x12\"\n\x06graphs\x18\t \x03(\x0b\x32\x12.ultipa.GraphAlias\x12\x1d\n\x06tables\x18\n \x03(\x0b\x32\r.ultipa.Table\x12!\n\nstatistics\x18\x0b \x01(\x0b\x32\r.ultipa.Table\x12)\n\x0c\x65xplain_plan\x18\x0c \x01(\x0b\x32\x13.ultipa.ExplainPlan\"7\n\tPathAlias\x12\x1b\n\x05paths\x18\x01 \x03(\x0b\x32\x0c.ultipa.Path\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"C\n\tEdgeAlias\x12\'\n\nedge_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"C\n\tNodeAlias\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"6\n\tAttrAlias\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\x1a\n\x04\x61ttr\x18\x02 \x01(\x0b\x32\x0c.ultipa.Attr\"9\n\nGraphAlias\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\x1c\n\x05graph\x18\x02 \x01(\x0b\x32\r.ultipa.Graph\"@\n\x04\x41ttr\x12(\n\nvalue_type\x18\x01 \x01(\x0e\x32\x14.ultipa.PropertyType\x12\x0e\n\x06values\x18\x02 \x03(\x0c\"\xc3\x01\n\x0c\x41ttrListData\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.ultipa.ResultType\x12\"\n\x05nodes\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\"\n\x05\x65\x64ges\x18\x03 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x1b\n\x05paths\x18\x04 \x03(\x0b\x32\x0c.ultipa.Path\x12\x1b\n\x05\x61ttrs\x18\x05 \x03(\x0b\x32\x0c.ultipa.Attr\x12\x0f\n\x07is_null\x18\x06 \x01(\x08\"E\n\x0b\x41ttrMapData\x12\x19\n\x03key\x18\x01 \x01(\x0b\x32\x0c.ultipa.Attr\x12\x1b\n\x05value\x18\x02 \x01(\x0b\x32\x0c.ultipa.Attr\"\x97\x01\n\x12UserSettingRequest\x12\x11\n\tuser_name\x18\x01 \x01(\t\x12.\n\x03opt\x18\x02 \x01(\x0e\x32!.ultipa.UserSettingRequest.OPTION\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\"\"\n\x06OPTION\x12\x0b\n\x07OPT_GET\x10\x00\x12\x0b\n\x07OPT_SET\x10\x01\"@\n\x10UserSettingReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\";\n\x15\x44ownloadFileRequestV2\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x0f\n\x07task_id\x18\x02 \x01(\t\"V\n\x11\x44ownloadFileReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x12\n\ntotal_size\x18\x02 \x01(\x05\x12\r\n\x05\x63hunk\x18\x03 \x01(\x0c\"j\n\rExportRequest\x12\x1f\n\x07\x64\x62_type\x18\x01 \x01(\x0e\x32\x0e.ultipa.DBType\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x19\n\x11select_properties\x18\x03 \x03(\t\x12\x0e\n\x06schema\x18\x04 \x01(\t\"\x7f\n\x0b\x45xportReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\'\n\nnode_table\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\'\n\nedge_table\x18\x03 \x01(\x0b\x32\x13.ultipa.EntityTable\"\x12\n\x10GetLeaderRequest\"0\n\x0eGetLeaderReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"\x8a\x01\n\x12InsertNodesRequest\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x0e\n\x06silent\x18\x02 \x01(\x08\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12\'\n\x0binsert_type\x18\x04 \x01(\x0e\x32\x12.ultipa.InsertType\"\xb2\x01\n\x10InsertNodesReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x11\n\ttime_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\x11\n\x05uuids\x18\x04 \x03(\x04\x42\x02\x30\x01\x12\x0b\n\x03ids\x18\x05 \x03(\t\x12\x16\n\x0eignore_indexes\x18\x06 \x03(\r\x12\x19\n\x11ignore_error_code\x18\x07 \x03(\r\"\xac\x01\n\x12InsertEdgesRequest\x12\'\n\nedge_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x0e\n\x06silent\x18\x02 \x01(\x08\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12 \n\x18\x63reate_node_if_not_exist\x18\x04 \x01(\x08\x12\'\n\x0binsert_type\x18\x05 \x01(\x0e\x32\x12.ultipa.InsertType\"\xa5\x01\n\x10InsertEdgesReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x11\n\ttime_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\x11\n\x05uuids\x18\x04 \x03(\x04\x42\x02\x30\x01\x12\x16\n\x0eignore_indexes\x18\x05 \x03(\r\x12\x19\n\x11ignore_error_code\x18\x06 \x03(\r\"C\n\x12InstallAlgoRequest\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x0b\n\x03md5\x18\x02 \x01(\t\x12\r\n\x05\x63hunk\x18\x03 \x01(\x0c\"2\n\x10InstallAlgoReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\")\n\x14UninstallAlgoRequest\x12\x11\n\talgo_name\x18\x01 \x01(\t\"4\n\x12UninstallAlgoReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"\x96\x01\n\x0fUploaderRequest\x12\x1f\n\x07\x64\x62_type\x18\x01 \x01(\x0e\x32\x0e.ultipa.DBType\x12\x12\n\ngraph_name\x18\x02 \x01(\t\x12\x19\n\x11total_file_counts\x18\x03 \x01(\x04\x12\x11\n\tfile_name\x18\x04 \x01(\t\x12\x11\n\tfile_size\x18\x05 \x01(\x04\x12\r\n\x05\x63hunk\x18\x06 \x01(\x0c\"/\n\rUploaderReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"M\n\x1c\x43reateGraphByUploaderRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x19\n\x11graph_description\x18\x02 \x01(\t\"<\n\x1a\x43reateGraphByUploaderReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"C\n\x12InstallExtaRequest\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x0b\n\x03md5\x18\x02 \x01(\t\x12\r\n\x05\x63hunk\x18\x03 \x01(\x0c\"2\n\x10InstallExtaReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\")\n\x14UninstallExtaRequest\x12\x11\n\texta_name\x18\x01 \x01(\t\"4\n\x12UninstallExtaReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"g\n\x06Status\x12%\n\nerror_code\x18\x01 \x01(\x0e\x32\x11.ultipa.ErrorCode\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12)\n\x0c\x63luster_info\x18\x03 \x01(\x0b\x32\x13.ultipa.ClusterInfo\"S\n\x0cRaftFollower\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04role\x18\x02 \x01(\x05\x12$\n\x06status\x18\x03 \x01(\x0e\x32\x14.ultipa.ServerStatus\"`\n\x0b\x43lusterInfo\x12\x10\n\x08redirect\x18\x01 \x01(\t\x12\x16\n\x0eleader_address\x18\x02 \x01(\t\x12\'\n\tfollowers\x18\x03 \x03(\x0b\x32\x14.ultipa.RaftFollower\"X\n\x04Path\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\'\n\nedge_table\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\"Y\n\x05Graph\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\'\n\nedge_table\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\"b\n\x05Table\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x1f\n\x07headers\x18\x02 \x03(\x0b\x32\x0e.ultipa.Header\x12$\n\ntable_rows\x18\x03 \x03(\x0b\x32\x10.ultipa.TableRow\"\x1a\n\x08TableRow\x12\x0e\n\x06values\x18\x01 \x03(\x0c\"V\n\x0b\x45ntityTable\x12\x1f\n\x07schemas\x18\x01 \x03(\x0b\x32\x0e.ultipa.Schema\x12&\n\x0b\x65ntity_rows\x18\x03 \x03(\x0b\x32\x11.ultipa.EntityRow\"\xab\x01\n\tEntityRow\x12\x10\n\x04uuid\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\n\n\x02id\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x15\n\tfrom_uuid\x18\x04 \x01(\x04\x42\x02\x30\x01\x12\x13\n\x07to_uuid\x18\x05 \x01(\x04\x42\x02\x30\x01\x12\x0f\n\x07\x66rom_id\x18\x06 \x01(\t\x12\r\n\x05to_id\x18\x07 \x01(\t\x12\x0e\n\x06values\x18\x08 \x03(\x0c\x12\x0f\n\x07is_null\x18\t \x01(\x08\"C\n\x06Schema\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12$\n\nproperties\x18\x02 \x03(\x0b\x32\x10.ultipa.Property\"w\n\x08Property\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12+\n\rproperty_type\x18\x02 \x01(\x0e\x32\x14.ultipa.PropertyType\x12\'\n\tsub_types\x18\x03 \x03(\x0e\x32\x14.ultipa.PropertyType\"L\n\x06Header\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12+\n\rproperty_type\x18\x02 \x01(\x0e\x32\x14.ultipa.PropertyType\"#\n\x05Value\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\x08ListData\x12\x0e\n\x06values\x18\x01 \x03(\x0c\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"*\n\x07SetData\x12\x0e\n\x06values\x18\x01 \x03(\x0c\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"&\n\x08MapValue\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\"<\n\x07MapData\x12 \n\x06values\x18\x01 \x03(\x0b\x32\x10.ultipa.MapValue\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"3\n\x0b\x45xplainPlan\x12$\n\nplan_nodes\x18\x01 \x03(\x0b\x32\x10.ultipa.PlanNode\"K\n\x08PlanNode\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\x14\n\x0c\x63hildren_num\x18\x02 \x01(\r\x12\x0b\n\x03uql\x18\x03 \x01(\t\x12\r\n\x05infos\x18\x04 \x01(\t\"J\n\x13\x41uthenticateRequest\x12&\n\x04type\x18\x01 \x01(\x0e\x32\x18.ultipa.AuthenticateType\x12\x0b\n\x03uql\x18\x02 \x01(\t\"3\n\x11\x41uthenticateReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status*\xa9\x01\n\nResultType\x12\x15\n\x11RESULT_TYPE_UNSET\x10\x00\x12\x14\n\x10RESULT_TYPE_PATH\x10\x01\x12\x14\n\x10RESULT_TYPE_NODE\x10\x02\x12\x14\n\x10RESULT_TYPE_EDGE\x10\x03\x12\x14\n\x10RESULT_TYPE_ATTR\x10\x04\x12\x15\n\x11RESULT_TYPE_TABLE\x10\x05\x12\x15\n\x11RESULT_TYPE_GRAPH\x10\x06*3\n\nInsertType\x12\n\n\x06NORMAL\x10\x00\x12\r\n\tOVERWRITE\x10\x01\x12\n\n\x06UPSERT\x10\x02*\xda\x01\n\x0cPropertyType\x12\t\n\x05UNSET\x10\x00\x12\t\n\x05INT32\x10\x01\x12\n\n\x06UINT32\x10\x02\x12\t\n\x05INT64\x10\x03\x12\n\n\x06UINT64\x10\x04\x12\t\n\x05\x46LOAT\x10\x05\x12\n\n\x06\x44OUBLE\x10\x06\x12\n\n\x06STRING\x10\x07\x12\x0c\n\x08\x44\x41TETIME\x10\x08\x12\r\n\tTIMESTAMP\x10\t\x12\x08\n\x04TEXT\x10\n\x12\x08\n\x04\x42LOB\x10\x0b\x12\t\n\x05POINT\x10\x0c\x12\x0b\n\x07\x44\x45\x43IMAL\x10\r\x12\x08\n\x04LIST\x10\x0e\x12\x07\n\x03SET\x10\x0f\x12\x07\n\x03MAP\x10\x10\x12\t\n\x05NULL_\x10\x11*\xb2\x02\n\tErrorCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x12\x0f\n\x0bPARAM_ERROR\x10\x02\x12\x11\n\rBASE_DB_ERROR\x10\x03\x12\x10\n\x0c\x45NGINE_ERROR\x10\x04\x12\x10\n\x0cSYSTEM_ERROR\x10\x05\x12\x11\n\rRAFT_REDIRECT\x10\x06\x12\x1f\n\x1bRAFT_LEADER_NOT_YET_ELECTED\x10\x07\x12\x12\n\x0eRAFT_LOG_ERROR\x10\x08\x12\r\n\tUQL_ERROR\x10\t\x12\x11\n\rNOT_RAFT_MODE\x10\n\x12\x1f\n\x1bRAFT_NO_AVAILABLE_FOLLOWERS\x10\x0b\x12\"\n\x1eRAFT_NO_AVAILABLE_ALGO_SERVERS\x10\x0c\x12\x15\n\x11PERMISSION_DENIED\x10\r*]\n\x0c\x46ollowerRole\x12\x0e\n\nROLE_UNSET\x10\x00\x12\x11\n\rROLE_READABLE\x10\x01\x12\x18\n\x14ROLE_ALGO_EXECUTABLE\x10\x02\x12\x10\n\x0cROLE_LEARNER\x10\x04*#\n\x0cServerStatus\x12\x08\n\x04\x44\x45\x41\x44\x10\x00\x12\t\n\x05\x41LIVE\x10\x01*.\n\x06\x44\x42Type\x12\n\n\x06\x44\x42NODE\x10\x00\x12\n\n\x06\x44\x42\x45\x44GE\x10\x01\x12\x0c\n\x08\x44\x42GLOBAL\x10\x02*w\n\x0bTASK_STATUS\x12\x10\n\x0cTASK_PENDING\x10\x00\x12\x12\n\x0eTASK_COMPUTING\x10\x01\x12\x10\n\x0cTASK_WRITING\x10\x02\x12\r\n\tTASK_DONE\x10\x03\x12\x0f\n\x0bTASK_FAILED\x10\x04\x12\x10\n\x0cTASK_STOPPED\x10\x05*\xfc\x02\n\x10\x41uthenticateType\x12\x17\n\x13PERMISSION_TYPE_UQL\x10\x00\x12\x1f\n\x1bPERMISSION_TYPE_INSERTNODES\x10\x01\x12\x1f\n\x1bPERMISSION_TYPE_INSERTEDGES\x10\x02\x12\x1a\n\x16PERMISSION_TYPE_EXPORT\x10\x03\x12 \n\x1cPERMISSION_TYPE_DOWNLOADFILE\x10\x04\x12\x1f\n\x1bPERMISSION_TYPE_INSTALLALGO\x10\x05\x12!\n\x1dPERMISSION_TYPE_UNINSTALLALGO\x10\x06\x12\x1c\n\x18PERMISSION_TYPE_UPLOADER\x10\x07\x12)\n%PERMISSION_TYPE_CREATEGRAPHBYUPLOADER\x10\x08\x12\x1f\n\x1bPERMISSION_TYPE_INSTALLEXTA\x10\t\x12!\n\x1dPERMISSION_TYPE_UNINSTALLEXTA\x10\n2\x8f\x02\n\nUltipaRpcs\x12\x42\n\x08SayHello\x12\x1a.ultipa.HelloUltipaRequest\x1a\x18.ultipa.HelloUltipaReply\"\x00\x12/\n\x03Uql\x12\x12.ultipa.UqlRequest\x1a\x10.ultipa.UqlReply\"\x00\x30\x01\x12\x45\n\x0bInsertNodes\x12\x1a.ultipa.InsertNodesRequest\x1a\x18.ultipa.InsertNodesReply\"\x00\x12\x45\n\x0bInsertEdges\x12\x1a.ultipa.InsertEdgesRequest\x1a\x18.ultipa.InsertEdgesReply\"\x00\x32\xb4\x07\n\x0eUltipaControls\x12\x42\n\x08SayHello\x12\x1a.ultipa.HelloUltipaRequest\x1a\x18.ultipa.HelloUltipaReply\"\x00\x12\x45\n\x0bUserSetting\x12\x1a.ultipa.UserSettingRequest\x1a\x18.ultipa.UserSettingReply\"\x00\x12?\n\tGetLeader\x12\x18.ultipa.GetLeaderRequest\x1a\x16.ultipa.GetLeaderReply\"\x00\x12\x31\n\x05UqlEx\x12\x12.ultipa.UqlRequest\x1a\x10.ultipa.UqlReply\"\x00\x30\x01\x12\x38\n\x06\x45xport\x12\x15.ultipa.ExportRequest\x1a\x13.ultipa.ExportReply\"\x00\x30\x01\x12N\n\x0e\x44ownloadFileV2\x12\x1d.ultipa.DownloadFileRequestV2\x1a\x19.ultipa.DownloadFileReply\"\x00\x30\x01\x12G\n\x0bInstallAlgo\x12\x1a.ultipa.InstallAlgoRequest\x1a\x18.ultipa.InstallAlgoReply\"\x00(\x01\x12K\n\rUninstallAlgo\x12\x1c.ultipa.UninstallAlgoRequest\x1a\x1a.ultipa.UninstallAlgoReply\"\x00\x12>\n\x08Uploader\x12\x17.ultipa.UploaderRequest\x1a\x15.ultipa.UploaderReply\"\x00(\x01\x12\x63\n\x15\x43reateGraphByUploader\x12$.ultipa.CreateGraphByUploaderRequest\x1a\".ultipa.CreateGraphByUploaderReply\"\x00\x12G\n\x0bInstallExta\x12\x1a.ultipa.InstallExtaRequest\x1a\x18.ultipa.InstallExtaReply\"\x00(\x01\x12K\n\rUninstallExta\x12\x1c.ultipa.UninstallExtaRequest\x1a\x1a.ultipa.UninstallExtaReply\"\x00\x12H\n\x0c\x41uthenticate\x12\x1b.ultipa.AuthenticateRequest\x1a\x19.ultipa.AuthenticateReply\"\x00\x62\x06proto3')

_RESULTTYPE = DESCRIPTOR.enum_types_by_name['ResultType']
ResultType = enum_type_wrapper.EnumTypeWrapper(_RESULTTYPE)
_INSERTTYPE = DESCRIPTOR.enum_types_by_name['InsertType']
InsertType = enum_type_wrapper.EnumTypeWrapper(_INSERTTYPE)
_PROPERTYTYPE = DESCRIPTOR.enum_types_by_name['PropertyType']
PropertyType = enum_type_wrapper.EnumTypeWrapper(_PROPERTYTYPE)
_ERRORCODE = DESCRIPTOR.enum_types_by_name['ErrorCode']
ErrorCode = enum_type_wrapper.EnumTypeWrapper(_ERRORCODE)
_FOLLOWERROLE = DESCRIPTOR.enum_types_by_name['FollowerRole']
FollowerRole = enum_type_wrapper.EnumTypeWrapper(_FOLLOWERROLE)
_SERVERSTATUS = DESCRIPTOR.enum_types_by_name['ServerStatus']
ServerStatus = enum_type_wrapper.EnumTypeWrapper(_SERVERSTATUS)
_DBTYPE = DESCRIPTOR.enum_types_by_name['DBType']
DBType = enum_type_wrapper.EnumTypeWrapper(_DBTYPE)
_TASK_STATUS = DESCRIPTOR.enum_types_by_name['TASK_STATUS']
TASK_STATUS = enum_type_wrapper.EnumTypeWrapper(_TASK_STATUS)
_AUTHENTICATETYPE = DESCRIPTOR.enum_types_by_name['AuthenticateType']
AuthenticateType = enum_type_wrapper.EnumTypeWrapper(_AUTHENTICATETYPE)
RESULT_TYPE_UNSET = 0
RESULT_TYPE_PATH = 1
RESULT_TYPE_NODE = 2
RESULT_TYPE_EDGE = 3
RESULT_TYPE_ATTR = 4
RESULT_TYPE_TABLE = 5
RESULT_TYPE_GRAPH = 6
NORMAL = 0
OVERWRITE = 1
UPSERT = 2
UNSET = 0
INT32 = 1
UINT32 = 2
INT64 = 3
UINT64 = 4
FLOAT = 5
DOUBLE = 6
STRING = 7
DATETIME = 8
TIMESTAMP = 9
TEXT = 10
BLOB = 11
POINT = 12
DECIMAL = 13
LIST = 14
SET = 15
MAP = 16
NULL_ = 17
SUCCESS = 0
FAILED = 1
PARAM_ERROR = 2
BASE_DB_ERROR = 3
ENGINE_ERROR = 4
SYSTEM_ERROR = 5
RAFT_REDIRECT = 6
RAFT_LEADER_NOT_YET_ELECTED = 7
RAFT_LOG_ERROR = 8
UQL_ERROR = 9
NOT_RAFT_MODE = 10
RAFT_NO_AVAILABLE_FOLLOWERS = 11
RAFT_NO_AVAILABLE_ALGO_SERVERS = 12
PERMISSION_DENIED = 13
ROLE_UNSET = 0
ROLE_READABLE = 1
ROLE_ALGO_EXECUTABLE = 2
ROLE_LEARNER = 4
DEAD = 0
ALIVE = 1
DBNODE = 0
DBEDGE = 1
DBGLOBAL = 2
TASK_PENDING = 0
TASK_COMPUTING = 1
TASK_WRITING = 2
TASK_DONE = 3
TASK_FAILED = 4
TASK_STOPPED = 5
PERMISSION_TYPE_UQL = 0
PERMISSION_TYPE_INSERTNODES = 1
PERMISSION_TYPE_INSERTEDGES = 2
PERMISSION_TYPE_EXPORT = 3
PERMISSION_TYPE_DOWNLOADFILE = 4
PERMISSION_TYPE_INSTALLALGO = 5
PERMISSION_TYPE_UNINSTALLALGO = 6
PERMISSION_TYPE_UPLOADER = 7
PERMISSION_TYPE_CREATEGRAPHBYUPLOADER = 8
PERMISSION_TYPE_INSTALLEXTA = 9
PERMISSION_TYPE_UNINSTALLEXTA = 10


_HELLOULTIPAREQUEST = DESCRIPTOR.message_types_by_name['HelloUltipaRequest']
_HELLOULTIPAREPLY = DESCRIPTOR.message_types_by_name['HelloUltipaReply']
_UQLREQUEST = DESCRIPTOR.message_types_by_name['UqlRequest']
_RESULTALIAS = DESCRIPTOR.message_types_by_name['ResultAlias']
_UQLREPLY = DESCRIPTOR.message_types_by_name['UqlReply']
_PATHALIAS = DESCRIPTOR.message_types_by_name['PathAlias']
_EDGEALIAS = DESCRIPTOR.message_types_by_name['EdgeAlias']
_NODEALIAS = DESCRIPTOR.message_types_by_name['NodeAlias']
_ATTRALIAS = DESCRIPTOR.message_types_by_name['AttrAlias']
_GRAPHALIAS = DESCRIPTOR.message_types_by_name['GraphAlias']
_ATTR = DESCRIPTOR.message_types_by_name['Attr']
_ATTRLISTDATA = DESCRIPTOR.message_types_by_name['AttrListData']
_ATTRMAPDATA = DESCRIPTOR.message_types_by_name['AttrMapData']
_USERSETTINGREQUEST = DESCRIPTOR.message_types_by_name['UserSettingRequest']
_USERSETTINGREPLY = DESCRIPTOR.message_types_by_name['UserSettingReply']
_DOWNLOADFILEREQUESTV2 = DESCRIPTOR.message_types_by_name['DownloadFileRequestV2']
_DOWNLOADFILEREPLY = DESCRIPTOR.message_types_by_name['DownloadFileReply']
_EXPORTREQUEST = DESCRIPTOR.message_types_by_name['ExportRequest']
_EXPORTREPLY = DESCRIPTOR.message_types_by_name['ExportReply']
_GETLEADERREQUEST = DESCRIPTOR.message_types_by_name['GetLeaderRequest']
_GETLEADERREPLY = DESCRIPTOR.message_types_by_name['GetLeaderReply']
_INSERTNODESREQUEST = DESCRIPTOR.message_types_by_name['InsertNodesRequest']
_INSERTNODESREPLY = DESCRIPTOR.message_types_by_name['InsertNodesReply']
_INSERTEDGESREQUEST = DESCRIPTOR.message_types_by_name['InsertEdgesRequest']
_INSERTEDGESREPLY = DESCRIPTOR.message_types_by_name['InsertEdgesReply']
_INSTALLALGOREQUEST = DESCRIPTOR.message_types_by_name['InstallAlgoRequest']
_INSTALLALGOREPLY = DESCRIPTOR.message_types_by_name['InstallAlgoReply']
_UNINSTALLALGOREQUEST = DESCRIPTOR.message_types_by_name['UninstallAlgoRequest']
_UNINSTALLALGOREPLY = DESCRIPTOR.message_types_by_name['UninstallAlgoReply']
_UPLOADERREQUEST = DESCRIPTOR.message_types_by_name['UploaderRequest']
_UPLOADERREPLY = DESCRIPTOR.message_types_by_name['UploaderReply']
_CREATEGRAPHBYUPLOADERREQUEST = DESCRIPTOR.message_types_by_name['CreateGraphByUploaderRequest']
_CREATEGRAPHBYUPLOADERREPLY = DESCRIPTOR.message_types_by_name['CreateGraphByUploaderReply']
_INSTALLEXTAREQUEST = DESCRIPTOR.message_types_by_name['InstallExtaRequest']
_INSTALLEXTAREPLY = DESCRIPTOR.message_types_by_name['InstallExtaReply']
_UNINSTALLEXTAREQUEST = DESCRIPTOR.message_types_by_name['UninstallExtaRequest']
_UNINSTALLEXTAREPLY = DESCRIPTOR.message_types_by_name['UninstallExtaReply']
_STATUS = DESCRIPTOR.message_types_by_name['Status']
_RAFTFOLLOWER = DESCRIPTOR.message_types_by_name['RaftFollower']
_CLUSTERINFO = DESCRIPTOR.message_types_by_name['ClusterInfo']
_PATH = DESCRIPTOR.message_types_by_name['Path']
_GRAPH = DESCRIPTOR.message_types_by_name['Graph']
_TABLE = DESCRIPTOR.message_types_by_name['Table']
_TABLEROW = DESCRIPTOR.message_types_by_name['TableRow']
_ENTITYTABLE = DESCRIPTOR.message_types_by_name['EntityTable']
_ENTITYROW = DESCRIPTOR.message_types_by_name['EntityRow']
_SCHEMA = DESCRIPTOR.message_types_by_name['Schema']
_PROPERTY = DESCRIPTOR.message_types_by_name['Property']
_HEADER = DESCRIPTOR.message_types_by_name['Header']
_VALUE = DESCRIPTOR.message_types_by_name['Value']
_LISTDATA = DESCRIPTOR.message_types_by_name['ListData']
_SETDATA = DESCRIPTOR.message_types_by_name['SetData']
_MAPVALUE = DESCRIPTOR.message_types_by_name['MapValue']
_MAPDATA = DESCRIPTOR.message_types_by_name['MapData']
_EXPLAINPLAN = DESCRIPTOR.message_types_by_name['ExplainPlan']
_PLANNODE = DESCRIPTOR.message_types_by_name['PlanNode']
_AUTHENTICATEREQUEST = DESCRIPTOR.message_types_by_name['AuthenticateRequest']
_AUTHENTICATEREPLY = DESCRIPTOR.message_types_by_name['AuthenticateReply']
_USERSETTINGREQUEST_OPTION = _USERSETTINGREQUEST.enum_types_by_name['OPTION']
HelloUltipaRequest = _reflection.GeneratedProtocolMessageType('HelloUltipaRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOULTIPAREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.HelloUltipaRequest)
  })
_sym_db.RegisterMessage(HelloUltipaRequest)

HelloUltipaReply = _reflection.GeneratedProtocolMessageType('HelloUltipaReply', (_message.Message,), {
  'DESCRIPTOR' : _HELLOULTIPAREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.HelloUltipaReply)
  })
_sym_db.RegisterMessage(HelloUltipaReply)

UqlRequest = _reflection.GeneratedProtocolMessageType('UqlRequest', (_message.Message,), {
  'DESCRIPTOR' : _UQLREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UqlRequest)
  })
_sym_db.RegisterMessage(UqlRequest)

ResultAlias = _reflection.GeneratedProtocolMessageType('ResultAlias', (_message.Message,), {
  'DESCRIPTOR' : _RESULTALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ResultAlias)
  })
_sym_db.RegisterMessage(ResultAlias)

UqlReply = _reflection.GeneratedProtocolMessageType('UqlReply', (_message.Message,), {
  'DESCRIPTOR' : _UQLREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UqlReply)
  })
_sym_db.RegisterMessage(UqlReply)

PathAlias = _reflection.GeneratedProtocolMessageType('PathAlias', (_message.Message,), {
  'DESCRIPTOR' : _PATHALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.PathAlias)
  })
_sym_db.RegisterMessage(PathAlias)

EdgeAlias = _reflection.GeneratedProtocolMessageType('EdgeAlias', (_message.Message,), {
  'DESCRIPTOR' : _EDGEALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EdgeAlias)
  })
_sym_db.RegisterMessage(EdgeAlias)

NodeAlias = _reflection.GeneratedProtocolMessageType('NodeAlias', (_message.Message,), {
  'DESCRIPTOR' : _NODEALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.NodeAlias)
  })
_sym_db.RegisterMessage(NodeAlias)

AttrAlias = _reflection.GeneratedProtocolMessageType('AttrAlias', (_message.Message,), {
  'DESCRIPTOR' : _ATTRALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrAlias)
  })
_sym_db.RegisterMessage(AttrAlias)

GraphAlias = _reflection.GeneratedProtocolMessageType('GraphAlias', (_message.Message,), {
  'DESCRIPTOR' : _GRAPHALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.GraphAlias)
  })
_sym_db.RegisterMessage(GraphAlias)

Attr = _reflection.GeneratedProtocolMessageType('Attr', (_message.Message,), {
  'DESCRIPTOR' : _ATTR,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Attr)
  })
_sym_db.RegisterMessage(Attr)

AttrListData = _reflection.GeneratedProtocolMessageType('AttrListData', (_message.Message,), {
  'DESCRIPTOR' : _ATTRLISTDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrListData)
  })
_sym_db.RegisterMessage(AttrListData)

AttrMapData = _reflection.GeneratedProtocolMessageType('AttrMapData', (_message.Message,), {
  'DESCRIPTOR' : _ATTRMAPDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrMapData)
  })
_sym_db.RegisterMessage(AttrMapData)

UserSettingRequest = _reflection.GeneratedProtocolMessageType('UserSettingRequest', (_message.Message,), {
  'DESCRIPTOR' : _USERSETTINGREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UserSettingRequest)
  })
_sym_db.RegisterMessage(UserSettingRequest)

UserSettingReply = _reflection.GeneratedProtocolMessageType('UserSettingReply', (_message.Message,), {
  'DESCRIPTOR' : _USERSETTINGREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UserSettingReply)
  })
_sym_db.RegisterMessage(UserSettingReply)

DownloadFileRequestV2 = _reflection.GeneratedProtocolMessageType('DownloadFileRequestV2', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADFILEREQUESTV2,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.DownloadFileRequestV2)
  })
_sym_db.RegisterMessage(DownloadFileRequestV2)

DownloadFileReply = _reflection.GeneratedProtocolMessageType('DownloadFileReply', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADFILEREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.DownloadFileReply)
  })
_sym_db.RegisterMessage(DownloadFileReply)

ExportRequest = _reflection.GeneratedProtocolMessageType('ExportRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExportRequest)
  })
_sym_db.RegisterMessage(ExportRequest)

ExportReply = _reflection.GeneratedProtocolMessageType('ExportReply', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExportReply)
  })
_sym_db.RegisterMessage(ExportReply)

GetLeaderRequest = _reflection.GeneratedProtocolMessageType('GetLeaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEADERREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.GetLeaderRequest)
  })
_sym_db.RegisterMessage(GetLeaderRequest)

GetLeaderReply = _reflection.GeneratedProtocolMessageType('GetLeaderReply', (_message.Message,), {
  'DESCRIPTOR' : _GETLEADERREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.GetLeaderReply)
  })
_sym_db.RegisterMessage(GetLeaderReply)

InsertNodesRequest = _reflection.GeneratedProtocolMessageType('InsertNodesRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSERTNODESREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertNodesRequest)
  })
_sym_db.RegisterMessage(InsertNodesRequest)

InsertNodesReply = _reflection.GeneratedProtocolMessageType('InsertNodesReply', (_message.Message,), {
  'DESCRIPTOR' : _INSERTNODESREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertNodesReply)
  })
_sym_db.RegisterMessage(InsertNodesReply)

InsertEdgesRequest = _reflection.GeneratedProtocolMessageType('InsertEdgesRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSERTEDGESREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertEdgesRequest)
  })
_sym_db.RegisterMessage(InsertEdgesRequest)

InsertEdgesReply = _reflection.GeneratedProtocolMessageType('InsertEdgesReply', (_message.Message,), {
  'DESCRIPTOR' : _INSERTEDGESREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertEdgesReply)
  })
_sym_db.RegisterMessage(InsertEdgesReply)

InstallAlgoRequest = _reflection.GeneratedProtocolMessageType('InstallAlgoRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLALGOREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallAlgoRequest)
  })
_sym_db.RegisterMessage(InstallAlgoRequest)

InstallAlgoReply = _reflection.GeneratedProtocolMessageType('InstallAlgoReply', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLALGOREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallAlgoReply)
  })
_sym_db.RegisterMessage(InstallAlgoReply)

UninstallAlgoRequest = _reflection.GeneratedProtocolMessageType('UninstallAlgoRequest', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLALGOREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallAlgoRequest)
  })
_sym_db.RegisterMessage(UninstallAlgoRequest)

UninstallAlgoReply = _reflection.GeneratedProtocolMessageType('UninstallAlgoReply', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLALGOREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallAlgoReply)
  })
_sym_db.RegisterMessage(UninstallAlgoReply)

UploaderRequest = _reflection.GeneratedProtocolMessageType('UploaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADERREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UploaderRequest)
  })
_sym_db.RegisterMessage(UploaderRequest)

UploaderReply = _reflection.GeneratedProtocolMessageType('UploaderReply', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADERREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UploaderReply)
  })
_sym_db.RegisterMessage(UploaderReply)

CreateGraphByUploaderRequest = _reflection.GeneratedProtocolMessageType('CreateGraphByUploaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEGRAPHBYUPLOADERREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.CreateGraphByUploaderRequest)
  })
_sym_db.RegisterMessage(CreateGraphByUploaderRequest)

CreateGraphByUploaderReply = _reflection.GeneratedProtocolMessageType('CreateGraphByUploaderReply', (_message.Message,), {
  'DESCRIPTOR' : _CREATEGRAPHBYUPLOADERREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.CreateGraphByUploaderReply)
  })
_sym_db.RegisterMessage(CreateGraphByUploaderReply)

InstallExtaRequest = _reflection.GeneratedProtocolMessageType('InstallExtaRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLEXTAREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallExtaRequest)
  })
_sym_db.RegisterMessage(InstallExtaRequest)

InstallExtaReply = _reflection.GeneratedProtocolMessageType('InstallExtaReply', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLEXTAREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallExtaReply)
  })
_sym_db.RegisterMessage(InstallExtaReply)

UninstallExtaRequest = _reflection.GeneratedProtocolMessageType('UninstallExtaRequest', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLEXTAREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallExtaRequest)
  })
_sym_db.RegisterMessage(UninstallExtaRequest)

UninstallExtaReply = _reflection.GeneratedProtocolMessageType('UninstallExtaReply', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLEXTAREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallExtaReply)
  })
_sym_db.RegisterMessage(UninstallExtaReply)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Status)
  })
_sym_db.RegisterMessage(Status)

RaftFollower = _reflection.GeneratedProtocolMessageType('RaftFollower', (_message.Message,), {
  'DESCRIPTOR' : _RAFTFOLLOWER,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.RaftFollower)
  })
_sym_db.RegisterMessage(RaftFollower)

ClusterInfo = _reflection.GeneratedProtocolMessageType('ClusterInfo', (_message.Message,), {
  'DESCRIPTOR' : _CLUSTERINFO,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ClusterInfo)
  })
_sym_db.RegisterMessage(ClusterInfo)

Path = _reflection.GeneratedProtocolMessageType('Path', (_message.Message,), {
  'DESCRIPTOR' : _PATH,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Path)
  })
_sym_db.RegisterMessage(Path)

Graph = _reflection.GeneratedProtocolMessageType('Graph', (_message.Message,), {
  'DESCRIPTOR' : _GRAPH,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Graph)
  })
_sym_db.RegisterMessage(Graph)

Table = _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {
  'DESCRIPTOR' : _TABLE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Table)
  })
_sym_db.RegisterMessage(Table)

TableRow = _reflection.GeneratedProtocolMessageType('TableRow', (_message.Message,), {
  'DESCRIPTOR' : _TABLEROW,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.TableRow)
  })
_sym_db.RegisterMessage(TableRow)

EntityTable = _reflection.GeneratedProtocolMessageType('EntityTable', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYTABLE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EntityTable)
  })
_sym_db.RegisterMessage(EntityTable)

EntityRow = _reflection.GeneratedProtocolMessageType('EntityRow', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYROW,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EntityRow)
  })
_sym_db.RegisterMessage(EntityRow)

Schema = _reflection.GeneratedProtocolMessageType('Schema', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Schema)
  })
_sym_db.RegisterMessage(Schema)

Property = _reflection.GeneratedProtocolMessageType('Property', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Property)
  })
_sym_db.RegisterMessage(Property)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Header)
  })
_sym_db.RegisterMessage(Header)

Value = _reflection.GeneratedProtocolMessageType('Value', (_message.Message,), {
  'DESCRIPTOR' : _VALUE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Value)
  })
_sym_db.RegisterMessage(Value)

ListData = _reflection.GeneratedProtocolMessageType('ListData', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ListData)
  })
_sym_db.RegisterMessage(ListData)

SetData = _reflection.GeneratedProtocolMessageType('SetData', (_message.Message,), {
  'DESCRIPTOR' : _SETDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.SetData)
  })
_sym_db.RegisterMessage(SetData)

MapValue = _reflection.GeneratedProtocolMessageType('MapValue', (_message.Message,), {
  'DESCRIPTOR' : _MAPVALUE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MapValue)
  })
_sym_db.RegisterMessage(MapValue)

MapData = _reflection.GeneratedProtocolMessageType('MapData', (_message.Message,), {
  'DESCRIPTOR' : _MAPDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MapData)
  })
_sym_db.RegisterMessage(MapData)

ExplainPlan = _reflection.GeneratedProtocolMessageType('ExplainPlan', (_message.Message,), {
  'DESCRIPTOR' : _EXPLAINPLAN,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExplainPlan)
  })
_sym_db.RegisterMessage(ExplainPlan)

PlanNode = _reflection.GeneratedProtocolMessageType('PlanNode', (_message.Message,), {
  'DESCRIPTOR' : _PLANNODE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.PlanNode)
  })
_sym_db.RegisterMessage(PlanNode)

AuthenticateRequest = _reflection.GeneratedProtocolMessageType('AuthenticateRequest', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATEREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AuthenticateRequest)
  })
_sym_db.RegisterMessage(AuthenticateRequest)

AuthenticateReply = _reflection.GeneratedProtocolMessageType('AuthenticateReply', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATEREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AuthenticateReply)
  })
_sym_db.RegisterMessage(AuthenticateReply)

_ULTIPARPCS = DESCRIPTOR.services_by_name['UltipaRpcs']
_ULTIPACONTROLS = DESCRIPTOR.services_by_name['UltipaControls']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INSERTNODESREPLY.fields_by_name['uuids']._options = None
  _INSERTNODESREPLY.fields_by_name['uuids']._serialized_options = b'0\001'
  _INSERTEDGESREPLY.fields_by_name['uuids']._options = None
  _INSERTEDGESREPLY.fields_by_name['uuids']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['uuid']._options = None
  _ENTITYROW.fields_by_name['uuid']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['from_uuid']._options = None
  _ENTITYROW.fields_by_name['from_uuid']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['to_uuid']._options = None
  _ENTITYROW.fields_by_name['to_uuid']._serialized_options = b'0\001'
  _RESULTTYPE._serialized_start=5128
  _RESULTTYPE._serialized_end=5297
  _INSERTTYPE._serialized_start=5299
  _INSERTTYPE._serialized_end=5350
  _PROPERTYTYPE._serialized_start=5353
  _PROPERTYTYPE._serialized_end=5571
  _ERRORCODE._serialized_start=5574
  _ERRORCODE._serialized_end=5880
  _FOLLOWERROLE._serialized_start=5882
  _FOLLOWERROLE._serialized_end=5975
  _SERVERSTATUS._serialized_start=5977
  _SERVERSTATUS._serialized_end=6012
  _DBTYPE._serialized_start=6014
  _DBTYPE._serialized_end=6060
  _TASK_STATUS._serialized_start=6062
  _TASK_STATUS._serialized_end=6181
  _AUTHENTICATETYPE._serialized_start=6184
  _AUTHENTICATETYPE._serialized_end=6564
  _HELLOULTIPAREQUEST._serialized_start=24
  _HELLOULTIPAREQUEST._serialized_end=58
  _HELLOULTIPAREPLY._serialized_start=60
  _HELLOULTIPAREPLY._serialized_end=127
  _UQLREQUEST._serialized_start=130
  _UQLREQUEST._serialized_end=262
  _RESULTALIAS._serialized_start=264
  _RESULTALIAS._serialized_end=333
  _UQLREPLY._serialized_start=336
  _UQLREPLY._serialized_end=746
  _PATHALIAS._serialized_start=748
  _PATHALIAS._serialized_end=803
  _EDGEALIAS._serialized_start=805
  _EDGEALIAS._serialized_end=872
  _NODEALIAS._serialized_start=874
  _NODEALIAS._serialized_end=941
  _ATTRALIAS._serialized_start=943
  _ATTRALIAS._serialized_end=997
  _GRAPHALIAS._serialized_start=999
  _GRAPHALIAS._serialized_end=1056
  _ATTR._serialized_start=1058
  _ATTR._serialized_end=1122
  _ATTRLISTDATA._serialized_start=1125
  _ATTRLISTDATA._serialized_end=1320
  _ATTRMAPDATA._serialized_start=1322
  _ATTRMAPDATA._serialized_end=1391
  _USERSETTINGREQUEST._serialized_start=1394
  _USERSETTINGREQUEST._serialized_end=1545
  _USERSETTINGREQUEST_OPTION._serialized_start=1511
  _USERSETTINGREQUEST_OPTION._serialized_end=1545
  _USERSETTINGREPLY._serialized_start=1547
  _USERSETTINGREPLY._serialized_end=1611
  _DOWNLOADFILEREQUESTV2._serialized_start=1613
  _DOWNLOADFILEREQUESTV2._serialized_end=1672
  _DOWNLOADFILEREPLY._serialized_start=1674
  _DOWNLOADFILEREPLY._serialized_end=1760
  _EXPORTREQUEST._serialized_start=1762
  _EXPORTREQUEST._serialized_end=1868
  _EXPORTREPLY._serialized_start=1870
  _EXPORTREPLY._serialized_end=1997
  _GETLEADERREQUEST._serialized_start=1999
  _GETLEADERREQUEST._serialized_end=2017
  _GETLEADERREPLY._serialized_start=2019
  _GETLEADERREPLY._serialized_end=2067
  _INSERTNODESREQUEST._serialized_start=2070
  _INSERTNODESREQUEST._serialized_end=2208
  _INSERTNODESREPLY._serialized_start=2211
  _INSERTNODESREPLY._serialized_end=2389
  _INSERTEDGESREQUEST._serialized_start=2392
  _INSERTEDGESREQUEST._serialized_end=2564
  _INSERTEDGESREPLY._serialized_start=2567
  _INSERTEDGESREPLY._serialized_end=2732
  _INSTALLALGOREQUEST._serialized_start=2734
  _INSTALLALGOREQUEST._serialized_end=2801
  _INSTALLALGOREPLY._serialized_start=2803
  _INSTALLALGOREPLY._serialized_end=2853
  _UNINSTALLALGOREQUEST._serialized_start=2855
  _UNINSTALLALGOREQUEST._serialized_end=2896
  _UNINSTALLALGOREPLY._serialized_start=2898
  _UNINSTALLALGOREPLY._serialized_end=2950
  _UPLOADERREQUEST._serialized_start=2953
  _UPLOADERREQUEST._serialized_end=3103
  _UPLOADERREPLY._serialized_start=3105
  _UPLOADERREPLY._serialized_end=3152
  _CREATEGRAPHBYUPLOADERREQUEST._serialized_start=3154
  _CREATEGRAPHBYUPLOADERREQUEST._serialized_end=3231
  _CREATEGRAPHBYUPLOADERREPLY._serialized_start=3233
  _CREATEGRAPHBYUPLOADERREPLY._serialized_end=3293
  _INSTALLEXTAREQUEST._serialized_start=3295
  _INSTALLEXTAREQUEST._serialized_end=3362
  _INSTALLEXTAREPLY._serialized_start=3364
  _INSTALLEXTAREPLY._serialized_end=3414
  _UNINSTALLEXTAREQUEST._serialized_start=3416
  _UNINSTALLEXTAREQUEST._serialized_end=3457
  _UNINSTALLEXTAREPLY._serialized_start=3459
  _UNINSTALLEXTAREPLY._serialized_end=3511
  _STATUS._serialized_start=3513
  _STATUS._serialized_end=3616
  _RAFTFOLLOWER._serialized_start=3618
  _RAFTFOLLOWER._serialized_end=3701
  _CLUSTERINFO._serialized_start=3703
  _CLUSTERINFO._serialized_end=3799
  _PATH._serialized_start=3801
  _PATH._serialized_end=3889
  _GRAPH._serialized_start=3891
  _GRAPH._serialized_end=3980
  _TABLE._serialized_start=3982
  _TABLE._serialized_end=4080
  _TABLEROW._serialized_start=4082
  _TABLEROW._serialized_end=4108
  _ENTITYTABLE._serialized_start=4110
  _ENTITYTABLE._serialized_end=4196
  _ENTITYROW._serialized_start=4199
  _ENTITYROW._serialized_end=4370
  _SCHEMA._serialized_start=4372
  _SCHEMA._serialized_end=4439
  _PROPERTY._serialized_start=4441
  _PROPERTY._serialized_end=4560
  _HEADER._serialized_start=4562
  _HEADER._serialized_end=4638
  _VALUE._serialized_start=4640
  _VALUE._serialized_end=4675
  _LISTDATA._serialized_start=4677
  _LISTDATA._serialized_end=4720
  _SETDATA._serialized_start=4722
  _SETDATA._serialized_end=4764
  _MAPVALUE._serialized_start=4766
  _MAPVALUE._serialized_end=4804
  _MAPDATA._serialized_start=4806
  _MAPDATA._serialized_end=4866
  _EXPLAINPLAN._serialized_start=4868
  _EXPLAINPLAN._serialized_end=4919
  _PLANNODE._serialized_start=4921
  _PLANNODE._serialized_end=4996
  _AUTHENTICATEREQUEST._serialized_start=4998
  _AUTHENTICATEREQUEST._serialized_end=5072
  _AUTHENTICATEREPLY._serialized_start=5074
  _AUTHENTICATEREPLY._serialized_end=5125
  _ULTIPARPCS._serialized_start=6567
  _ULTIPARPCS._serialized_end=6838
  _ULTIPACONTROLS._serialized_start=6841
  _ULTIPACONTROLS._serialized_end=7789
# @@protoc_insertion_point(module_scope)
